import logging
import gzip
from pathlib import Path
from experimaestro import Task, Param, Annotated, pathgenerator, Option
import datamaestro_text.data.ir as ir
from datamaestro_text.utils.shuffle import shuffle


def getpathname(context, config):
    name = "triplets.lst"
    if config.compressed:
        name = "triplets.lst.gz"

    return context.currentpath() / name


class ShuffledTrainingTripletsLines(Task):
    """Shuffle a set of training triplets"""

    data: Param[ir.TrainingTriplets]
    path: Annotated[Path, pathgenerator(getpathname)]
    seed: Param[int]
    compressed: Option[bool] = True

    def taskoutputs(self):
        return ir.TrainingTripletsLines(
            id="", path=self.path, ids=self.data.ids, sep="\t"
        )

    def execute(self):
        # --- Shuffle using the shuf command with a seed

        def triplegenerator():
            logging.info("Starting to output triples")
            count = 0

            for qid, doca, docb in self.data.iter():
                yield f"{qid}\t{doca}\t{docb}\n"
                count += 1

            logging.info("Triples output ended (%d triples)", count)

        logging.info("Creating generator")

        # Output can be a stream or nothing
        if self.compressed:
            output = gzip.open(self.path, "wt")
        else:
            output = self.path.open("wt")

        with output:
            shuffle(triplegenerator(), output)



from pathlib import Path


def read_sv(file, sep):
    if hasattr(file, "read"):
        for line in file:
            # TODO: handle values with sep, blank values, etc?
            yield line.rstrip("\r\n").split(sep)
    else:
        _open = open
        path = Path(file)

        if path.suffix.endswith(".gz"):
            import gzip

            _open = gzip.open

        with _open(file, "rt") as f:
            yield from read_sv(f, sep)


def write_sv(file, data, sep):
    if hasattr(file, "write"):
        for values in data:
            # TODO: handle values with sep, blank values, etc?
            file.write(sep.join(map(str, values)) + "\n")
        file.flush()
    else:
        with open(file, "wt") as f:
            write_sv(f, data, sep)


def read_tsv(file):
    return read_sv(file, sep="\t")


def write_tsv(file, data):
    write_sv(file, data, sep="\t")



# See documentation on http://experimaestro.github.io/datamaestro/

from datamaestro.data import File
from datamaestro.data.ml import Supervised
from datamaestro.definitions import datatasks, datatags, dataset
from datamaestro.download.single import filedownloader
from datamaestro.utils import HashCheck


@datatags("unsupervised")
@datatasks("information extraction")
@filedownloader(
    "train.json",
    "https://github.com/thunlp/FewRel/raw/master/data/train_wiki.json",
    checker=HashCheck("5e663e9c3f1bfbdb2de72696e9504fd7"),
)
@filedownloader(
    "validation.json",
    "https://github.com/thunlp/FewRel/raw/master/data/val_wiki.json",
    checker=HashCheck("3f25573428c0332cb64b367a275ab0c7"),
)
@dataset(
    Supervised, url="https://thunlp.github.io/1/fewrel1.html",
)
def v1(train, validation):
    """FewRel 1.0 - a Few-shot Relation classification dataset

   FewRel is a Few-shot Relation classification dataset, which features 70, 000 natural
   language sentences expressing 100 relations annotated by crowdworkers.

   Only the train and validation dataset are available. The test set is hidden
   for the leaderboard.
  """
    return {"train": File(path=train), "validation": File(path=validation)}



from dataclasses import dataclass
from pathlib import Path
from typing import List, NamedTuple, Optional
import re

from datamaestro_text.data.ir import (
    AdhocAssessedTopic,
    AdhocAssessment,
    AdhocTopic,
)

# --- Assessments


def parse_qrels(path: Path):
    with path.open("rt") as fp:
        _qid = None
        assessments = []

        for line in fp:
            qid, _, docno, rel = re.split(r"\s+", line.strip())
            if qid != _qid:
                if _qid is not None:
                    yield AdhocAssessedTopic(_qid, assessments)
                _qid = qid
                assessments = []
            assessments.append(AdhocAssessment(docno, int(rel)))

        yield AdhocAssessedTopic(_qid, assessments)


# ---- TOPICS


@dataclass()
class TrecAdhocTopic(AdhocTopic):
    description: str
    narrative: str


def cleanup(s: Optional[str]) -> str:
    return s.replace("\t", " ").strip() if s is not None else ""


def parse_query_format(file, xml_prefix=None):
    """Parse TREC XML query format"""
    if xml_prefix is None:
        xml_prefix = ""

    if hasattr(file, "read"):
        num, title, desc, narr, reading = None, None, None, None, None
        for line in file:
            if line.startswith("**"):
                # translation comment in older formats (e.g., TREC 3 Spanish track)
                continue
            elif line.startswith("</top>"):
                if num:
                    yield TrecAdhocTopic(
                        num, cleanup(title), {}, cleanup(desc), cleanup(narr)
                    )
                num, title, desc, narr, reading = None, None, None, None, None
            elif line.startswith("<num>"):
                num = line[len("<num>") :].replace("Number:", "").strip()
                reading = None
            elif line.startswith(f"<{xml_prefix}title>"):
                title = line[len(f"<{xml_prefix}title>") :].strip()
                if title == "":
                    reading = "title"
                else:
                    reading = None
            elif line.startswith(f"<{xml_prefix}desc>"):
                desc = ""
                reading = "desc"
            elif line.startswith(f"<{xml_prefix}narr>"):
                narr = ""
                reading = "narr"
            elif reading == "desc":
                desc += line.strip() + " "
            elif reading == "narr":
                narr += line.strip() + " "
            elif reading == "title":
                title += line.strip() + " "
    else:
        with open(file, "rt") as f:
            yield from parse_query_format(f)



import datamaestro


class Repository(datamaestro.Repository):
    AUTHOR = """Benjamin Piwowarski <benjamin@piwowarski.fr>"""
    DESCRIPTION = """Text datasets repository"""
    NAMESPACE = """text"""



from datamaestro.definitions import argument, datatasks, datatags, dataset
from datamaestro.download.single import filedownloader
from datamaestro_text.data.text import TextFile
from datamaestro.utils import HashCheck


@filedownloader(
    "file",
    "https://oscar-public.huma-num.fr/shuffled/en_dedup.txt.gz",
    checker=HashCheck("5c906ede3c5265f8934b62c275a754bc"),
)
@dataset(TextFile, url="https://oscar-corpus.com/", size="2.3T")
def english(file):
    """Huge French corpus from INRIA

  OSCAR or Open Super-large Crawled ALMAnaCH coRpus is a huge multilingual corpus
  obtained by language classification and filtering of the Common Crawl corpus using
  the goclassy architecture.
  """
    return {"path": file}



from logging import Logger
import inspect
import logging
import os
from pathlib import Path
import tempfile
from threading import Thread
from typing import BinaryIO, Callable, TextIO, Union


class StreamGenerator(Thread):
    """Create a FIFO pipe (*nix only) that is fed by the provider generator"""

    def __init__(self, generator: Callable[[Union[TextIO, BinaryIO]], None], mode="wb"):
        super().__init__()
        tmpdir = tempfile.mkdtemp()
        self.mode = mode
        self.filepath = Path(os.path.join(tmpdir, "fifo.json"))
        os.mkfifo(self.filepath)
        self.generator = generator
        self.error = False

    def run(self):
        try:
            with self.filepath.open(self.mode) as out:
                try:
                    self.generator(out)
                except Exception:
                    # Just write something so the file is closed
                    if isinstance(out, TextIO):
                        out.write("")
                    else:
                        out.write(b"0")
                    raise
        except Exception:
            self.error = True
            raise

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, *args):
        self.join()
        self.filepath.unlink()
        self.filepath.parent.rmdir()
        if self.error:
            raise AssertionError("Error with the generator")


class Handler:
    """Returns a result that depends on the type of the argument

    Example:
    ```
    handler = Handler()

    @handler()
    def trectopics(topics: TrecAdhocTopics):
        return ("-topicreader", "Trec", "-topics", topics.path)

    @handler()
    def tsvtopics(topics: ir_csv.AdhocTopics):
        return ("-topicreader", "TsvInt", "-topics", topics.path)

    command.extend(handler[topics])

    ```
    """

    def __init__(self):
        self.handlers = {}
        self.defaulthandler = None

    def default(self):
        assert self.defaulthandler is None

        def annotate(method):
            self.defaulthandler = method
            return method

        return annotate

    def __call__(self):
        def annotate(method):
            spec = inspect.getfullargspec(method)
            assert len(spec.args) == 1 and spec.varargs is None

            self.handlers[spec.annotations[spec.args[0]]] = method

        return annotate

    def __getitem__(self, key):
        handler = self.handlers.get(key.__class__, None)
        if handler is None:
            if self.default is None:
                raise RuntimeError(
                    f"No handler for {key.__class__} and no default handler"
                )
            handler = self.defaulthandler

        return handler(key)


def easylog():
    """
    Returns a logger with the caller's __name__
    """
    import inspect

    try:
        frame = inspect.stack()[1]  # caller
        module = inspect.getmodule(frame[0])
        return Logger(module.__name__)
    except IndexError:
        return Logger("UNKNOWN")


class EasyLogger:
    @property
    def logger(self):
        clsdict = self.__class__.__dict__

        logger = clsdict.get("__LOGGER__", None)
        if logger is None:
            logger = logging.getLogger(self.__class__.__qualname__)
            self.__class__.__LOGGER__ = logger

        return logger



"""The AQUAINT Corpus, Linguistic Data Consortium (LDC) catalog number LDC2002T31 and ISBN 1-58563-240-6 consists of newswire text data in English, drawn from three sources: the Xinhua News Service (People's Republic of China), the New York Times News Service, and the Associated Press Worldstream News Service. It was prepared by the LDC for the AQUAINT Project, and will be used in official benchmark evaluations conducted by National Institute of Standards and Technology (NIST)."""

from datamaestro.definitions import DatafolderPath
from datamaestro.data import Base
from datamaestro_text.data.ir.trec import TipsterCollection
from datamaestro.definitions import argument, datatasks, datatags, dataset
from datamaestro.download.links import links, linkfolder


URL = "https://catalog.ldc.upenn.edu/LDC2002T31"


@linkfolder("documents", [DatafolderPath("edu.upenn.ldc.aquaint", "APW")])
@dataset(TipsterCollection, url=URL, id="apw")
def apw(documents):
    """Associated Press (1998-2000)"""
    return {"path": documents}


@linkfolder("documents", [DatafolderPath("edu.upenn.ldc.aquaint", "NYT")])
@dataset(TipsterCollection, url=URL, id="nyt")
def nyt(documents):
    """New York Times (1998-2000)"""
    return {"path": documents}


@linkfolder("documents", [DatafolderPath("edu.upenn.ldc.aquaint", "XIE")])
@dataset(TipsterCollection, url=URL, id="xie")
def xie(documents):
    """Xinhua News Agency newswires (1996-2000)"""
    return {"path": documents}


@links("documents", apw=apw.path, nyt=nyt.path, xie=xie.path)
@dataset(TipsterCollection, url=URL, id="")
def aquaint(documents):
    """Aquaint documents"""
    return {"path": documents}



"""
GloVe is an unsupervised learning algorithm for obtaining vector representations for words.
  Training is performed on aggregated global word-word co-occurrence statistics from a corpus,
  and the resulting representations showcase interesting linear substructures of the word vector space.
"""

from datamaestro.definitions import dataset
from datamaestro.data import Base, Generic
from datamaestro.download import reference
from datamaestro.download.archive import zipdownloader
from datamaestro.download.single import filedownloader
from datamaestro_text.data.embeddings import WordEmbeddingsText

# size: 822M
# statistics:
#   tokens: 6G
#   vocabulary: 400K
#   cased: false
@zipdownloader("embeddings", "http://nlp.stanford.edu/data/glove.6B.zip")
@dataset(WordEmbeddingsText, id="6b")
def glove_6b(embeddings):
    """Embeddings for 6B words in various dimensions"""
    return {"path": embeddings}


@reference("data_6b", glove_6b)
@dataset(WordEmbeddingsText, id="6b.50")
def glove_6b_50(data_6b):
    """Glove 6B - dimension 50"""
    return {"path": data_6b.path / "glove.6B.50d.txt"}


@reference("data_6b", glove_6b)
@dataset(WordEmbeddingsText, id="6b.100")
def glove_6b_100(data_6b):
    """Glove 6B - dimension 100"""
    return {"path": data_6b.path / "glove.6B.100d.txt"}


@reference("data_6b", glove_6b)
@dataset(WordEmbeddingsText, id="6b.200")
def glove_6b_200(data_6b):
    """Glove 6B - dimension 200"""
    return {"path": data_6b.path / "glove.6B.200d.txt"}


...


@reference("data_6b", glove_6b)
@dataset(WordEmbeddingsText, id="6b.300")
def glove_6b_300(data_6b):
    """Glove 6B - dimension 200"""
    return {"path": data_6b.path / "glove.6B.200d.txt"}


@filedownloader("embeddings", "http://nlp.stanford.edu/data/glove.42B.300d.zip")
@dataset(WordEmbeddingsText, id="42b")
# size: 2.03G
# statistics:
#   cased: true
#   tokens: 42B
#   vocabulary: 2.2M
#   dimension: 300
def glove_42b(embeddings):
    """Glove embeddings trained on Common Crawl with 42B tokens"""
    return {"path": embeddings}


@filedownloader("embeddings", "http://nlp.stanford.edu/data/glove.840B.300d.zip")
@dataset(WordEmbeddingsText, id="840b")
# size: 2.03G
# statistics:
#   cased: true
#   tokens: 840G
#   vocabulary: 2.2M
#   dimension: 300
def glove_840b(embeddings):
    """Glove embeddings trained on Common Crawl with 840B tokens"""
    return {"path": embeddings}



# See documentation on http://experimaestro.github.io/datamaestro/

from datamaestro.definitions import argument, datatasks, datatags, dataset
from datamaestro_text.data.text import TextFolder
from datamaestro.download.archive import tardownloader
from datamaestro.utils import HashCheck


@datatags("text", "books", "English")
@datatasks("language modeling")
@tardownloader(
    "folder",
    "https://storage.googleapis.com/huggingface-nlp/datasets/bookcorpus/bookcorpus.tar.bz2",
    checker=HashCheck("5c906ede3c5265f8934b62c275a754bc"),
)
@dataset(TextFolder, id="", url="https://yknzhu.wixsite.com/mbweb", size="4.3G")
def main(folder):
    """Unpublished books from Smashwords

    The books are concatened in two files hosted on huggingface NLP storage.
    Each sentence is on a separate line and tokens are space separated.
    """
    return {"path": folder}



"""Generic data types for information retrieval"""

from pathlib import Path
from typing import Callable, Dict, Iterator, List, Optional, Tuple
import random
from experimaestro import Config
from datamaestro.definitions import argument, datatasks, Param, Meta
from dataclasses import dataclass
from datamaestro.data import Base, documentation


@dataclass()
class AdhocTopic:
    """The most generic topic: an ID with some text"""

    qid: str
    """Query identifier"""

    text: str
    """The main query text"""

    metadata: Dict[str, str]
    """Extra-information about the query"""


@dataclass(frozen=True)
class AdhocAssessment:
    """Adhoc assessments associate a document ID with a relevance"""

    #: Document identifier
    docno: str

    #: Relevance (> 0 if relevant)
    rel: float


@dataclass()
class AdhocAssessedTopic:
    qid: str
    assessments: List[AdhocAssessment]


@dataclass()
class AdhocDocument:
    """A document with an identifier"""

    docid: str
    text: str
    internal_docid: Optional[int] = None


class AdhocDocuments(Base):
    """A set of documents with identifiers"""

    count: Meta[Optional[int]]
    """Number of documents"""

    def iter(self) -> Iterator[AdhocDocument]:
        """(deprecated, use iter_documents) Returns an iterator over adhoc documents"""
        raise NotImplementedError("No document iterator")

    def iter_documents(self) -> Iterator[AdhocDocument]:
        return self.iter()

    def iter_ids(self) -> Iterator[str]:
        """Iterates over document ids

        By default, use iter_documents, which is not really efficient.
        """
        for doc in self.iter():
            yield doc.docid

    @property
    def documentcount(self):
        """Returns the number of terms in the index"""
        raise NotImplementedError()


class AdhocDocumentStore(AdhocDocuments):
    """A document store

    A document store can
    - match external/internal ID
    - return the document content
    - return the number of documents
    """

    def document_text(self, docid: str) -> str:
        """Returns the text of the document given its id"""
        raise NotImplementedError(f"document_text() for {self.__class__}")

    def docid_internal2external(self, docid: int):
        """Converts an internal collection ID (integer) to an external ID"""
        raise NotImplementedError()

    def document(self, internal_docid: int) -> AdhocDocument:
        """Returns a document given its internal ID"""
        docid = self.docid_internal2external(internal_docid)
        return AdhocDocument(
            docid, self.document_text(docid), internal_docid=internal_docid
        )

    def iter_sample(
        self, randint: Optional[Callable[[int], int]]
    ) -> Iterator[AdhocDocument]:
        """Sample documents from the dataset"""
        length = self.documentcount
        randint = randint or (lambda max: random.randint(0, max - 1))
        while True:
            yield self.document(randint(length))


class AdhocIndex(AdhocDocumentStore):
    """An index can be used to retrieve documents based on terms"""

    @property
    def termcount(self):
        """Returns the number of terms in the index"""
        raise NotImplementedError()

    def term_df(self, term: str):
        """Returns the document frequency"""
        raise NotImplementedError()


class AdhocTopics(Base):
    def iter(self) -> Iterator[AdhocTopic]:
        """Returns an iterator over topics"""
        raise NotImplementedError()

    def count(self) -> Optional[int]:
        """Returns the number of topics if known"""
        return None


class AdhocAssessments(Base):
    """Ad-hoc assessements (qrels)"""

    def iter(self) -> Iterator[AdhocAssessedTopic]:
        """Returns an iterator over assessments"""
        raise NotImplementedError()


class AdhocRun(Base):
    """IR adhoc run"""

    pass


@datatasks("information retrieval")
class Adhoc(Base):
    """An Adhoc IR collection"""

    documents: Param[AdhocDocuments]
    """The set of documents"""

    topics: Param[AdhocTopics]
    """The set of topics"""

    assessments: Param[AdhocAssessments]
    """The set of assessments (for each topic)"""


class RerankAdhoc(Adhoc):
    """Re-ranking ad-hoc task based on an existing run"""

    run: Param[AdhocRun]
    """The run to re-rank"""


class Measure(Config):
    """An Information Retrieval measure"""

    pass


class TrainingTriplets(Base):
    """Triplet for training IR systems: query / query ID, positive document, negative document

    attributes:

        ids: True if the triplet is made of IDs, False otherwise
    """

    ids: Meta[bool]

    def iter(self) -> Iterator[Tuple[str, str, str]]:
        raise NotImplementedError()


def autoopen(path: Path, mode: str):
    print("Reading", path)
    if path.suffix == ".gz":
        import gzip

        return gzip.open(path, mode)
    return path.open(mode)


class TrainingTripletsLines(TrainingTriplets):
    """Training triplets with one line per triple (text only)"""

    sep: Meta[str]
    path: Param[Path]

    def iter(self) -> Iterator[Tuple[str, str, str]]:
        with autoopen(self.path, "rt") as fp:
            for line in fp:
                q, pos, neg = line.strip().split(self.sep)
                yield q, pos, neg



from pathlib import Path
from datamaestro.data import Base, File, argument
from datamaestro.definitions import datatags
import numpy as np
from typing import Tuple, List


@datatags("word embeddings")
class WordEmbeddings(Base):
    """Generic word embeddings"""

    def load(self) -> Tuple[List[str], np.matrix]:
        """Load the word embeddings

        Returns:
            The word dictionary and the matrix
        """
        raise NotImplementedError()


@argument("encoding", str, ignored=True, default="utf-8")
class WordEmbeddingsText(WordEmbeddings, File):
    """Word embeddings as a text word / values"""

    def load(self):
        words = []
        vectors = []
        dimension = None
        with self.path.open("rt", encoding=self.encoding) as fp:
            for ix, line in enumerate(fp):
                word, *values = line.split(" ")
                vectors.append([float(x) for x in values])
                if dimension is None:
                    dimension = len(values)
                else:
                    assert dimension == len(values)
                words.append(word)
        return words, np.matrix(vectors)



# Copyright (c) 2016 Salle, Alexandre <atsalle@inf.ufrgs.br>
# Author: Salle, Alexandre <atsalle@inf.ufrgs.br>
# Modified: Benjamin Piwowarski <b@piwowarski.fr>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of
# this software and associated documentation files (the "Software"), to deal in
# the Software without restriction, including without limitation the rights to
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
# the Software, and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import os
import tempfile
import random
import logging
from typing import TextIO

# need better way to estimate this given overhead for str. setting to
# 0.5GB uses rougly 4GB of ram.
MEMORY = int(float(os.environ.get("MEMORY", 4.0)) / 8 * 1024 ** 3)


def shuffle_and_close(buf, f):
    random.shuffle(buf)
    f.writelines(buf)
    f.close()


def shuffle(input, output: TextIO, memory=MEMORY):
    """Shuffle using temporary file
    """
    files = []
    files.append(tempfile.NamedTemporaryFile(mode="w", delete=False))
    total_bytes = 0
    total_lines = 0
    buf = []
    bytes_used = 0

    for line in input:
        bytes_used += len(line)
        total_bytes += len(line)
        total_lines += 1
        buf.append(line)
        if bytes_used >= memory:
            shuffle_and_close(buf, files[-1])
            files.append(tempfile.NamedTemporaryFile(mode="w", delete=False))
            buf = []
            bytes_used = 0
    if buf:
        shuffle_and_close(buf, files[-1])

    buf = []
    avg_bytes_per_line = total_bytes / float(total_lines)
    logging.info("Output from %d temporary files", len(files))
    files = [open(f.name) for f in files]
    while files:
        rm_files = []
        lines_per_file = int((memory / avg_bytes_per_line) / len(files))
        for f in files:
            lines = f.readlines(lines_per_file)
            if not lines:
                rm_files.append(f)
            buf += lines
        random.shuffle(buf)
        output.writelines(buf)
        buf = []
        for f in rm_files:
            files.remove(f)
            os.unlink(f.name)



from datamaestro.data import Base, File, argument
import datamaestro.data.csv as csv


@argument("ratings", type=File)
class RatedItems(Base):
    pass


@argument("links", type=csv.Generic)
@argument("movies", type=csv.Generic)
@argument("tags", type=csv.Generic)
class Movielens(RatedItems):
    pass



from datamaestro.definitions import dataset
from datamaestro.data.ml import Supervised
from datamaestro_text.data.tagging import CoNLL_U
from datamaestro.download.archive import zipdownloader

# --- gsd


# description: |

# tasks: [ "POS parsing", "dependency parsing" ]
# tags: [ "french" ]

# files:
#   readme: README.md
#   stats: stats.xml
#   train: !@tagging:CoNLL_U
#     path: fr_gsd-ud-train.conllu
#   dev: !@tagging:CoNLL_U
#     path: fr_gsd-ud-dev.conllu
#   test: !@tagging:CoNLL_U
#     path: fr_gsd-ud-test.conllu


@zipdownloader(
    "ds", "https://codeload.github.com/UniversalDependencies/UD_French-GSD/zip/master"
)
@dataset(url="https://github.com/UniversalDependencies/UD_French-GSD")
def gsd(ds) -> Supervised:
    """French GSD

    The UD_French-GSD was converted in 2015 from the content head version of the
    universal dependency treebank v2.0 (https://github.com/ryanmcd/uni-dep-tb). It
    is updated since 2015 independently from the previous source.
    """
    return {
        "train": CoNLL_U(path=ds / "fr_gsd-ud-train.conllu"),
        "test": CoNLL_U(path=ds / "fr_gsd-ud-dev.conllu"),
        "validation": CoNLL_U(path=ds / "fr_gsd-ud-test.conllu"),
    }


# --- partut

# # See documentation on http://experimaestro.github.io/datamaestro/

# # Configuration
# # See http://experimaestro.github.io/datamaestro/configuration/

# name: UD_French-ParTUT
# web: https://github.com/UniversalDependencies/UD_French-ParTUT

# description: |
#   UD_French-ParTUT is a conversion of a multilingual parallel treebank developed
#   at the University of Turin, and consisting of a variety of text genres,
#   including talks, legal texts and Wikipedia articles, among others.

# tasks: [ "POS parsing", "dependency parsing" ]
# tags: [ "french" ]


# # Describe how to download
# # See http://experimaestro.github.io/datamaestro/download/

# download: !@/archive:Zip
#   url: https://codeload.github.com/UniversalDependencies/UD_French-ParTUT/zip/master

# files:
#   readme: README.md
#   stats: stats.xml
#   train: !@tagging:CoNLL_U
#     path: fr_partut-ud-train.conllu
#   dev: !@tagging:CoNLL_U
#     path: fr_partut-ud-dev.conllu
#   test: !@tagging:CoNLL_U
#     path: fr_partut-ud-test.conllu



from datamaestro.data.ml import FolderBased, Supervised
from datamaestro.definitions import datatasks, datatags, dataset
from datamaestro.download.archive import tardownloader


@tardownloader("data", "http://ai.stanford.edu/~amaas/data/sentiment/aclImdb_v1.tar.gz")
@dataset(Supervised, url="http://ai.stanford.edu/~amaas/data/sentiment/", id="")
def aclimdb(data):
    """Large Movie Review Dataset

  Paper http://ai.stanford.edu/~amaas/papers/wvSent_acl2011.pdf
  """
    return {
        "train": FolderBased(path=data / "train", classes=["neg", "pos"]),
        "test": FolderBased(path=data / "test", classes=["neg", "pos"]),
    }



from csv import DictReader
from typing import Iterator

from datamaestro.data import File, documentation
from datamaestro_text.data.ir import (
    AdhocDocument,
    AdhocDocuments,
    AdhocTopic,
    AdhocTopics,
)
from datamaestro.data.csv import Generic as GenericCSV
import xml.etree.ElementTree as ET


class Topics(AdhocTopics, File):
    """XML format used in Adhoc topics"""

    def iter(self) -> Iterator[AdhocTopic]:
        """Returns an iterator over topics"""
        tree = ET.parse(self.path)
        for topic in tree.findall("topic"):
            yield AdhocTopic(
                topic.get("number"),
                topic.find("query").text,
                {
                    "question": topic.find("question").text,
                    "narrative": topic.find("narrative").text,
                },
            )


class Documents(AdhocDocuments, GenericCSV):
    @documentation
    def iter(self) -> Iterator[AdhocDocument]:
        """Returns an iterator over adhoc documents"""
        with self.path.open("r") as fp:
            for row in DictReader(fp):
                yield AdhocDocument(
                    row["cord_uid"], f"""{row["title"]} {row["abstract"]}"""
                )



from datamaestro.data import Base, File
from datamaestro.definitions import (
    datatasks,
    datatags,
    dataset,
    metadataset,
)
from datamaestro.download.archive import zipdownloader
from datamaestro_text.data.text import TrainingText


@datatags("text")
@datatasks("language modeling")
@metadataset(TrainingText)
def WikiText(data, type):
    """WikiText-2

    WikiText language modeling dataset is a collection of over 100 million
    tokens extracted from the set of verified Good and Featured articles on
    Wikipedia. The dataset is available under the Creative Commons
    Attribution-ShareAlike License.

    Compared to the preprocessed version of Penn Treebank (PTB), WikiText-2 is over
    2 times larger and WikiText-103 is over 110 times larger. The WikiText dataset
    also features a far larger vocabulary and retains the original case, punctuation
    and numbers - all of which are removed in PTB. As it is composed of full
    articles, the dataset is well suited for models that can take advantage of long
    term dependencies.

    https://blog.einstein.ai/the-wikitext-long-term-dependency-language-modeling-dataset/
    """
    return {
        "train": File(path=data / ("wiki.train.%s" % type)),
        "validation": File(path=data / ("wiki.valid.%s" % type)),
        "test": File(path=data / ("wiki.test.%s" % type)),
    }


@zipdownloader(
    "data", "https://s3.amazonaws.com/research.metamind.io/wikitext/wikitext-2-v1.zip"
)
@dataset(WikiText, id="2.tokens")
def wikitext_2_words(data):
    """The small wikitext corpus, already tokenized"""
    return WikiText(data, "tokens")


@zipdownloader(
    "data",
    "https://s3.amazonaws.com/research.metamind.io/wikitext/wikitext-2-raw-v1.zip",
)
@dataset(WikiText, id="2.raw")
def wikitext_2_raw(data):
    """The small wikitext corpus (raw data)"""
    return WikiText(data, "raw")


@zipdownloader(
    "data", "https://s3.amazonaws.com/research.metamind.io/wikitext/wikitext-103-v1.zip"
)
@dataset(WikiText, id="103.tokens")
def wikitext_103_words(data):
    return WikiText(data, "tokens")


@zipdownloader(
    "data",
    "https://s3.amazonaws.com/research.metamind.io/wikitext/wikitext-103-raw-v1.zip",
)
@dataset(WikiText, id="103.raw")
def wikitext_103_raw(data):
    return WikiText(data, "raw")
